/**
 * Generic overlord prototype
 */

function overlord (name) {

    /** Unique identifier, set at creation or by deserialize() */
    this.id = name;

    /** Goal the overlord follows */
    this.goal = undefined;

    this.underlings = {
        "spawns": [],  // list of spawn names
        "creeps": [],  // list of creep names (NOT ids!) This is because creeps have no ID while being spawned.
    };

    /** Serializes the overlord state so that it can survive in memory */
    this.serialize = function() {
        var data = {
            "id": this.id,
            "goal": this.goal,
            "underlings": this.underlings,
        };
        Memory.overlords[this.id] = data;
        console.log("Overlord: Wrote overlord "+this.id+" to memory!");
    };

    /** Reconstructs the overlord state from memory given its identifier */
    this.deserialize = function() {
        this.goal = Memory.overlords[this.id]["goal"];
        this.underlings = Memory.overlords[this.id]["underlings"];

        console.log("Overlord: Read overlord "+this.id+" from memory!");
    };
    
    /**
     * Helper function to add a spawn to the underlings of this overlord
     * @param {string} spawn_name 
     */
    this.add_spawn = function(spawn_name) {
        let spawn = Game.spawns[spawn_name];
        if (spawn.memory.owner !== undefined) { // spawn already owned by someone
            throw "Spawn already taken!";
        }
        spawn.memory.owner = this.id;
        this.underlings["spawns"].push(spawn_name);
        console.log("Overlord: assigned spawn \""+spawn_name+"\" to overlord \""+this.id+"\"!");
    };

    /**
     * Helper function to add a creep to the underlings of this overlord
     * @param {string} creep_name 
     */
    this.add_creep = function(creep_name) {
        let creep = Game.creeps[creep_name];
        if (creep.memory.owner !== undefined && creep.memory.owner !== this.id) { // creep already owned by someone
            console.log("Overlord: ERROR Creep was already taken!");
            throw "Creep already taken!";
        }
        creep.memory.owner = this.id;
        this.underlings["creeps"].push(creep_name);
        console.log("Overlord: assigned creep \""+creep_name+"\" to overlord \""+this.id+"\"!");
    }

    /**
     * run function to be implemented by non-generic overlords
     */
    //this.run = function () {console.log("Trying to run a generic overlord! Bad!");};

};

module.exports = overlord;